import datetime
import time

from .test_utils import *
from tlbx.logging_utils import dbg
from tlbx.email_utils import *


def test_read_email():
    criteria = ["SINCE", datetime.date.today().strftime("%d-%b-%Y")]
    msgs = read_email(
        criteria=criteria,
        limit=2,
        host=test_config["IMAPHost"],
        username=test_config["IMAPUsername"],
        password=test_config["IMAPPassword"],
    )
    for msg in msgs:
        dbg(msg)
        payload = extract_email_payload(msg)
        dbg(payload)


def test_send_email():
    msg = create_email(
        test_config["SMTPUsername"],
        test_config["EmailDestination"],
        subject="Test Send Email: %s" % time.time(),
        body="Test body",
        html="<p>Alternative HTML</p>",
        attachments=["textfile.txt"],  # TODO: use pkg_resource
    )
    send_email(
        msg,
        host=test_config["SMTPHost"],
        port=test_config["SMTPPort"],
        username=test_config["SMTPUsername"],
        password=test_config["SMTPPassword"],
    )



import configparser
import logging
import os

assert "TLBX_CONFIG" in os.environ, (
    "Please specify the location of a tlbx config file using the "
    "environment variable TLBX_CONFIG"
)

config = configparser.ConfigParser()
config.read(os.environ["TLBX_CONFIG"])
test_config = config["TEST"]

logging.getLogger().setLevel(logging.DEBUG)



import logging
from pprint import pformat, pprint as pp
import sys

import sqlparse as sp

from tlbx.object_utils import get_class_vars


class FontSpecialChars:
    ENDC = "\033[0m"


class FontColors:
    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"
    NONE = ""  # Will use default terminal coloring


RESERVED_COLORS = [
    "RED",  # errors
    "YELLOW",  # warnings
    "BLACK",  # to avoid conflicts with terminal defaults
    "WHITE",  # to avoid conflicts with terminal defaults
]

COLOR_OPTIONS = [x for x in get_class_vars(FontColors) if x not in RESERVED_COLORS]


class FontEffects:
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"
    INVERTED = "\033[7m"


logging.basicConfig(format="%(message)s")


def get_logging_level():
    return logging.getLogger().getEffectiveLevel()


def sqlformat(sql):
    return sp.format(sql, reindent=True, keyword_case="upper")


def format_msg(
    msg, label="parent", indent=0, color=None, autocolor=False, format_func=pformat
):
    if isinstance(format_func, str):
        format_func = globals()[format_func]

    if format_func and format_func != pformat:
        msg = format_func(msg)
        if label:
            msg = "\n" + msg
    elif not isinstance(msg, str):
        msg = pformat(msg)
        if label:
            msg = "\n" + msg

    if indent is not None and int(indent):
        msg = (" " * int(indent)) + msg

    if label:
        if label == "parent":
            label = sys._getframe().f_back.f_code.co_name
        msg = label.strip() + ": " + msg

    if (not color) and autocolor:
        assert label, "No label provided, can not use autocolor"
        color_index = ord(label[0]) % len(COLOR_OPTIONS)
        color = COLOR_OPTIONS[color_index]

    if color:
        msg = getattr(FontColors, color.upper()) + msg + FontSpecialChars.ENDC

    return msg


def log(msg, label="parent", **kwargs):
    if get_logging_level() > logging.INFO:
        return
    if label == "parent":
        label = sys._getframe().f_back.f_code.co_name
    msg = format_msg(msg, label=label, autocolor=True, **kwargs)
    logging.info(msg)


def dbg(msg, label="parent", **kwargs):
    if get_logging_level() > logging.DEBUG:
        return
    if label == "parent":
        label = sys._getframe().f_back.f_code.co_name
    msg = format_msg(msg, label=label, autocolor=True, **kwargs)
    logging.debug(msg)


def dbgsql(msg, label="parent", **kwargs):
    if get_logging_level() > logging.DEBUG:
        return
    if label == "parent":
        label = sys._getframe().f_back.f_code.co_name
    msg = format_msg(msg, label=label, autocolor=True, format_func=sqlformat, **kwargs)
    logging.debug(msg)


def warn(msg, label="parent", **kwargs):
    if get_logging_level() > logging.WARNING:
        return
    if label == "parent":
        label = sys._getframe().f_back.f_code.co_name
    msg = format_msg(msg, label=label, color="yellow", **kwargs)
    logging.warning(msg)


def error(msg, label="parent", **kwargs):
    if get_logging_level() > logging.ERROR:
        return
    if label == "parent":
        label = sys._getframe().f_back.f_code.co_name
    msg = format_msg(msg, label=label, color="red", **kwargs)
    logging.error(msg)


class PrintMixin:
    repr_attrs = []

    def __repr__(self):
        if self.repr_attrs:
            return "<%s %s>" % (
                type(self).__name__,
                " ".join(
                    [
                        "%s=%s" % (field, getattr(self, field))
                        for field in self.repr_attrs
                    ]
                ),
            )
        return "<%s %s>" % (type(self).__name__, id(self))

    def __str__(self):
        return str(vars(self))



import os
import subprocess


def rmfile(filename, ignore_missing=False):
    try:
        os.remove(filename)
    except FileNotFoundError:
        if ignore_missing:
            return
        raise


def shell(cmd, shell=True, capture_output=True, **kwargs):
    return subprocess.run(cmd, shell=shell, capture_output=capture_output, **kwargs)



from unittest.mock import patch

from .test_utils import *
from tlbx import Script, Arg


@Script(Arg("name", help="A name"))
def script(name):
    print("Name:", name)


def test_script():
    with patch("argparse._sys.argv", ["test_script_decorator.py", "test"]):
        script()
